@@ -18,5 +18,11 @@ extern int nvmf_disconnect(const char *desc, int argc, char **argv);
 extern int nvmf_disconnect_all(const char *desc, int argc, char **argv);
 extern int nvmf_config(const char *desc, int argc, char **argv);
 extern int nvmf_dim(const char *desc, int argc, char **argv);
+extern nvme_ctrl_t nvmf_create_discover_ctrl(nvme_root_t r, nvme_host_t h,
+					     struct nvme_fabrics_config *cfg,
+					     struct tr_config *trcfg);
+extern char *nvmf_get_default_trsvcid(const char *transport,
+				      bool discovery_ctrl);
+
 
 #endif
