@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 #undef CMD_INC_FILE
 #define CMD_INC_FILE plugins/nvidia/nvidia-nvme
 
@@ -6,7 +7,7 @@
 
 #include "cmd.h"
 
-PLUGIN(NAME("nvidia", "NVIDIA vendor specific extensions"),
+PLUGIN(NAME("nvidia", "NVIDIA vendor specific extensions", NVME_VERSION),
 	COMMAND_LIST(
 		ENTRY("id-ctrl", "Send NVMe Identify Controller", id_ctrl)
 	)
